@@ -3,40 +3,51 @@
 {-# LANGUAGE ScopedTypeVariables #-}
 
 module Program.Mighty.Report (
-    Reporter
-  , initReporter
-  , report
-  , reportDo
-  , warpHandler
-  , printStdout
-  ) where
+    Reporter,
+    initReporter,
+    report,
+    reportDo,
+    warpHandler,
+    printStdout,
+) where
 
 #if __GLASGOW_HASKELL__ < 709
 import Control.Applicative
 #endif
+import Control.Exception
+import qualified Control.Exception as E (catch)
 import Control.Monad
 import Data.ByteString (ByteString)
 import qualified Data.ByteString.Char8 as BS
 import Data.UnixTime
-import GHC.IO.Exception (IOErrorType(..))
+import GHC.IO.Exception (IOErrorType (..))
 import Network.Wai
 import Network.Wai.Handler.Warp (InvalidRequest)
+import System.Exit (ExitCode)
 import System.IO
 import System.IO.Error (ioeGetErrorType)
 import System.Posix (getProcessID)
-import UnliftIO.Exception
+
+import Network.HTTP2.Client (HTTP2Error)
+#ifdef HTTP_OVER_TLS
+import Network.TLS (TLSException)
+import Network.Wai.Handler.WarpTLS (WarpTLSException)
+#ifdef HTTP_OVER_QUIC
+import Network.QUIC (QUICException)
+#endif
+#endif
 
 import Program.Mighty.ByteString
 
-data Method = FileOnly | FileAndStdout deriving Eq
+data Method = FileOnly | FileAndStdout deriving (Eq)
 data Reporter = Reporter Method FilePath
 
 initReporter :: Bool -> FilePath -> Reporter
 initReporter debug reportFile = Reporter method reportFile
   where
     method
-      | debug     = FileAndStdout
-      | otherwise = FileOnly
+        | debug = FileAndStdout
+        | otherwise = FileOnly
 
 report :: Reporter -> ByteString -> IO ()
 report (Reporter method reportFile) msg = handle (\(SomeException _) -> return ()) $ do
@@ -49,18 +60,29 @@ report (Reporter method reportFile) msg = handle (\(SomeException _) -> return (
 ----------------------------------------------------------------
 
 reportDo :: Reporter -> IO () -> IO ()
-reportDo rpt act = act `catchAny` warpHandler rpt Nothing
+reportDo rpt act = act `E.catch` warpHandler rpt Nothing
 
 ----------------------------------------------------------------
 
+{- FOURMOLU_DISABLE -}
 warpHandler :: Reporter -> Maybe Request -> SomeException -> IO ()
 warpHandler rpt _ se
-  | Just (_ :: InvalidRequest) <- fromException se = return ()
-  | Just (e :: IOException)    <- fromException se =
-        if ioeGetErrorType e `elem` [ResourceVanished,InvalidArgument]
-        then return ()
-        else report rpt $ bshow se
-  | otherwise = report rpt $ bshow se
+    | Just (_ :: ExitCode)         <- fromException se = return ()
+    | Just (e :: IOException)      <- fromException se =
+        if ioeGetErrorType e `elem` [ResourceVanished, InvalidArgument]
+            then return ()
+            else report rpt $ bshow se
+    | Just (_ :: InvalidRequest)   <- fromException se = return () -- Warp
+    | Just (_ :: HTTP2Error)       <- fromException se = return ()
+#ifdef HTTP_OVER_TLS
+    | Just (_ :: TLSException)     <- fromException se = return ()
+    | Just (_ :: WarpTLSException) <- fromException se = return ()
+#ifdef HTTP_OVER_QUIC
+    | Just (_ :: QUICException)    <- fromException se = return ()
+#endif
+#endif
+    | otherwise = report rpt $ bshow se
+{- FOURMOLU_ENABLE -}
 
 ----------------------------------------------------------------
 
