@@ -1,13 +1,13 @@
 {-# LANGUAGE CPP #-}
 
 module Program.Mighty.Resource (
-    amIrootUser
-  , setGroupUser
-  , unlimit
-  ) where
+    amIrootUser,
+    setGroupUser,
+    unlimit,
+) where
 
+import Control.Exception
 import System.Posix
-import UnliftIO.Exception
 
 ----------------------------------------------------------------
 
@@ -18,17 +18,21 @@ amIrootUser = (== 0) <$> getRealUserID
 ----------------------------------------------------------------
 
 -- | Setting user and group.
-setGroupUser :: String -- ^ User
-             -> String -- ^ Group
-             -> IO Bool
+setGroupUser
+    :: String
+    -- ^ User
+    -> String
+    -- ^ Group
+    -> IO Bool
 setGroupUser user group = do
     root <- amIrootUser
-    if root then do
-        getGroupEntryForName group >>= setGroupID . groupID
-        getUserEntryForName user >>= setUserID . userID
-        return True
-      else
-        return False
+    if root
+        then do
+            getGroupEntryForName group >>= setGroupID . groupID
+            getUserEntryForName user >>= setUserID . userID
+            return True
+        else
+            return False
 
 ----------------------------------------------------------------
 
@@ -36,8 +40,10 @@ setGroupUser user group = do
 unlimit :: Integer -> IO ()
 unlimit limit = handle (\(SomeException _) -> return ()) $ do
     hard <- hardLimit <$> getResourceLimit ResourceOpenFiles
-    let lim = if hard == ResourceLimitInfinity then
-                  ResourceLimits (ResourceLimit limit) hard
+    let lim =
+            if hard == ResourceLimitInfinity
+                then
+                    ResourceLimits (ResourceLimit limit) hard
                 else
-                  ResourceLimits hard hard
+                    ResourceLimits hard hard
     setResourceLimit ResourceOpenFiles lim
