@@ -1,12 +1,13 @@
 module Program.Mighty.Signal (
-  -- * Signals
-    sigStop
-  , sigReload
-  , sigRetire
-  , sigInfo
-  -- * Signal handling
-  , setHandler
-  ) where
+    -- * Signals
+    sigStop,
+    sigReload,
+    sigRetire,
+    sigInfo,
+
+    -- * Signal handling
+    setHandler,
+) where
 
 import Control.Monad
 import System.Posix
@@ -15,7 +16,7 @@ import System.Posix
 
 -- | The signal to stop Mighty.
 sigStop :: Signal
-sigStop   = sigTERM
+sigStop = sigTERM
 
 -- | The signal to reload a configration file.
 sigReload :: Signal
@@ -27,7 +28,7 @@ sigRetire = sigQUIT
 
 -- | The signal to get information from Mighty.
 sigInfo :: Signal
-sigInfo   = sigUSR2
+sigInfo = sigUSR2
 
 ----------------------------------------------------------------
 
