@@ -1,10 +1,15 @@
-{-# LANGUAGE OverloadedStrings, CPP #-}
+{-# LANGUAGE CPP #-}
+{-# LANGUAGE OverloadedStrings #-}
 
 module Main where
 
-#ifndef HTTP_OVER_TLS
+#ifndef HTTP_OVER_QUIC
 import Control.Monad (when)
 #endif
+
+#ifdef DHALL
+import Data.List (isSuffixOf)
+#endif
 import Data.Version (showVersion)
 import Network.Wai.Application.Classic hiding ((</>))
 import System.Directory (getCurrentDirectory)
@@ -34,17 +39,18 @@ main = do
     checkTLS opt
     let reportFile = reportFileName opt
         debug = opt_debug_mode opt
-    rpt <- initReporter debug reportFile >>= checkReporter reportFile
-    let run = server opt rpt route
-    if debug then run else background opt run
+        rpt = initReporter debug reportFile
+        run = server opt rpt route
+    if debug then run id else run (background opt)
   where
     getOptRoute = getArgs >>= eachCase
     svrnm = programName ++ "/" ++ programVersion
     eachCase args
       | n == 0 = do
           root <- amIrootUser
-          let opt | root      = (defaultOption svrnm) { opt_port = 80 }
-                  | otherwise = defaultOption svrnm
+          let opt0 = defaultOption { opt_server_name = svrnm }
+          let opt | root      = opt0 { opt_port = 80 }
+                  | otherwise = opt0
           dir <- getCurrentDirectory
           let dst = fromString . addTrailingPathSeparator $ dir
               route = [Block ["*"] [RouteFile "/" dst]]
@@ -52,9 +58,18 @@ main = do
       | n == 2 = do
           let config_file = args !! 0
           routing_file <- getAbsoluteFile (args !! 1)
-          opt   <- parseOption config_file svrnm
+#ifdef DHALL
+          let isDhall = ".dhall" `Data.List.isSuffixOf` config_file
+          opt   <- if isDhall
+                   then parseOptionDhall config_file
+                   else parseOption config_file
+#else
+          opt   <- parseOption config_file
+#endif
           route <- parseRoute  routing_file defaultDomain defaultPort
-          let opt' = opt {opt_routing_file = Just routing_file}
+          let opt' = opt { opt_routing_file = Just routing_file
+                         , opt_server_name = svrnm
+                         }
           return (opt',route)
       | otherwise = do
           hPutStrLn stderr "Usage: mighty"
@@ -67,13 +82,14 @@ main = do
       | otherwise       = do
           dir <- getCurrentDirectory
           return $ dir </> normalise file
-    checkReporter _          (Right rpt) = return rpt
-    checkReporter reportFile (Left e)    = do
-        hPutStrLn stderr $ reportFile ++ " is not writable"
-        hPrint stderr e
-        exitFailure
 #ifdef HTTP_OVER_TLS
+#ifdef HTTP_OVER_QUIC
     checkTLS _ = return ()
+#else
+    checkTLS opt = when (opt_service opt > 2) $ do
+        hPutStrLn stderr "This mighty does not support QUIC"
+        exitFailure
+#endif
 #else
     checkTLS opt = when (opt_service opt > 1) $ do
         hPutStrLn stderr "This mighty does not support TLS"
@@ -84,12 +100,10 @@ main = do
 
 background :: Option -> IO () -> IO ()
 background opt svr = do
-    putStrLn $ "Serving on port " ++ show port ++ " and detaching this terminal..."
+    putStrLn $ "Detaching this terminal..."
     putStrLn $ "(If errors occur, they will be written in \"" ++ reportFileName opt ++ "\".)"
     hFlush stdout
     daemonize svr
-  where
-    port = opt_port opt
 
 reportFileName :: Option -> FilePath
 reportFileName opt
