@@ -6,17 +6,22 @@ module Server (server, defaultDomain, defaultPort) where
 
 import Control.Concurrent (runInUnboundThread)
 import Control.Exception (try)
-import Control.Monad (unless, when)
-import Data.Either (fromRight)
+import Control.Monad (unless)
 import qualified Data.ByteString.Char8 as BS
-import Data.Streaming.Network (bindPortTCP)
+import Data.Either (fromRight)
+import Data.Streaming.Network (bindPortTCP, bindPortUDP)
+#if __GLASGOW_HASKELL__ >= 906
+import GHC.Conc.Sync
+import Data.List (sort)
+import Data.Maybe (fromMaybe)
+#endif
 import qualified Network.HTTP.Client as H
 import Network.Socket (Socket, close)
+import Network.Wai
 import Network.Wai.Application.Classic hiding ((</>))
 import Network.Wai.Handler.Warp
 import Network.Wai.Logger
 import System.Exit (ExitCode(..), exitSuccess)
-import System.IO
 import System.IO.Error (ioeGetErrorString)
 import System.Posix (exitImmediately, Handler(..), getProcessID, setFileMode)
 import System.Posix.Signals (sigCHLD)
@@ -31,23 +36,15 @@ import Data.Char (isSpace)
 import Data.List (dropWhileEnd)
 import Network.TLS (Credentials(..),SessionManager)
 import qualified Network.TLS as TLS
-import Network.TLS.SessionManager
-import qualified Network.TLS.SessionManager as SM
+import qualified Network.TLS.SessionTicket as SM
 import Network.Wai.Handler.WarpTLS
 #ifdef HTTP_OVER_QUIC
-import Control.Concurrent.Async (mapConcurrently_)
 import Data.Bits
 import Data.ByteString (ByteString)
 import Data.List (find)
 import Data.Maybe (fromJust)
 import qualified Network.QUIC.Internal as Q
 import Network.Wai.Handler.WarpQUIC
-#ifdef DROP_EXCEPT_BIND
-import Control.Monad (forM_)
-import Foreign.C.Types (CInt(..))
-import System.Directory (listDirectory)
-import System.Posix.Signals (sigUSR1)
-#endif
 #endif
 #else
 data Credentials
@@ -77,38 +74,37 @@ type LogRemover = IO ()
 
 ----------------------------------------------------------------
 
-server :: Option -> Reporter -> RouteDB -> IO ()
-server opt rpt route = reportDo rpt $ do
+server :: Option -> Reporter -> RouteDB -> (IO () -> IO ()) -> IO ()
+server opt rpt route runMighty = reportDo rpt $ do
+    labelMe "Mighty main"
     unlimit openFileNumber
     svc <- openService opt
-    unless debug writePidFile
-    rdr <- newRouteDBRef route
-    tmgr <- T.initialize (naturalToInt (opt_connection_timeout opt) * 1000000)
-#ifdef HTTP_OVER_TLS
-    mcred <- Just <$> loadCredentials opt
-    smgr <- Just <$> SM.newSessionManager SM.defaultConfig { dbMaxSize = 1000 }
-#else
-    let mcred = Nothing
-        smgr = Nothing
-#endif
-    _changed <- setGroupUser (opt_user opt) (opt_group opt)
-#ifdef DROP_EXCEPT_BIND
-    when _changed dropExceptBind
-#endif
-    logCheck logtype
-    (zdater,_) <- clockDateCacher
-    ap <- initLogger FromSocket logtype zdater
-    let lgr = apacheLogger ap
-        remover = logRemover ap
-        pushlgr = serverpushLogger ap
-    mgr <- getManager opt
-    setHandlers opt rpt svc remover rdr
+    runMighty $ do
+        unless debug writePidFile
+        rdr <- newRouteDBRef route
+        let usec = naturalToInt (opt_connection_timeout opt) * 1000000
+        T.withManager usec $ \tmgr -> do
+            (mcred, smgr) <- setup opt
+            _changed <- setGroupUser (opt_user opt) (opt_group opt)
+            logCheck logtype
+            -- "Time cacher of FastLogger (AutoUpdate)"
+            (zdater,_) <- clockDateCacher
+            -- Loggerset of FastLogger (Debounce)
+            ap <- initLogger FromSocket logtype zdater
+            let lgr = apacheLogger ap
+                remover = logRemover ap
+                pushlgr = serverpushLogger ap
+            -- HTTP Client Manager
+            mgr <- getManager opt
+            setHandlers opt rpt svc remover rdr
 
-    report rpt "Mighty started"
-    runInUnboundThread $ mighty opt rpt svc lgr pushlgr mgr rdr mcred smgr tmgr
-    report rpt "Mighty retired"
-    remover
-    exitSuccess
+            report rpt "Mighty started"
+            runInUnboundThread $ do
+                labelMe "Mighty main (bound thread)"
+                mighty opt rpt svc lgr pushlgr mgr rdr mcred smgr tmgr
+            report rpt "Mighty retired"
+            remover
+            exitSuccess
   where
     debug = opt_debug_mode opt
     port = opt_port opt
@@ -145,12 +141,29 @@ setHandlers opt rpt svc remover rdr = do
     retireHandler = Catch $ do
         report rpt "Mighty retiring"
         closeService svc -- this lets warp break
-    infoHandler = Catch $ report rpt "obsoleted"
+    infoHandler = Catch $ do
+        labelMe "Info signale handler"
+        threadSummary >>= mapM_ (report rpt . showT)
+    showT (i, l, s) = BS.pack (i ++ " " ++ l ++ ": " ++ show s)
     reloadHandler = Catch $ do
         ifRouteFileIsValid rpt opt $ \newroute -> do
             writeRouteDBRef rdr newroute
             report rpt "Mighty reloaded"
 
+#if __GLASGOW_HASKELL__ >= 906
+threadSummary :: IO [(String, String, ThreadStatus)]
+threadSummary = (sort <$> listThreads) >>= mapM summary
+  where
+    summary t = do
+        let idstr = drop 9 $ show t
+        l <- fromMaybe "(no name)" <$> threadLabel t
+        s <- threadStatus t
+        return (idstr, l, s)
+#else
+threadSummary :: IO [(String, String, String)]
+threadSummary = return []
+#endif
+
 #ifdef HTTP_OVER_TLS
 loadCredentials :: Option -> IO Credentials
 loadCredentials opt = do
@@ -163,9 +176,9 @@ loadCredentials opt = do
     strip = dropWhileEnd isSpace . dropWhile isSpace
     split "" = []
     split s = case break (',' ==) s of
-      ("",r)  -> split (tail r)
+      ("",r)  -> split $ drop 1 r
       (s',"") -> [s']
-      (s',r)  -> s' : split (tail r)
+      (s',r)  -> s' : split (drop 1 r)
     chain_files = map strip $ split $ opt_tls_chain_files opt
 #endif
 
@@ -174,9 +187,9 @@ loadCredentials opt = do
 ifRouteFileIsValid :: Reporter -> Option -> (RouteDB -> IO ()) -> IO ()
 ifRouteFileIsValid rpt opt act = case opt_routing_file opt of
     Nothing    -> return ()
-    Just rfile -> try (parseRoute rfile defaultDomain defaultPort) >>= either reportError act
+    Just rfile -> try (parseRoute rfile defaultDomain defaultPort) >>= either reportError_ act
   where
-    reportError = report rpt . BS.pack . ioeGetErrorString
+    reportError_ = report rpt . BS.pack . ioeGetErrorString
 
 ----------------------------------------------------------------
 
@@ -187,14 +200,14 @@ mighty :: Option -> Reporter -> Service
        -> IO ()
 mighty opt rpt svc lgr pushlgr mgr rdr _mcreds _msmgr tmgr
   = reportDo rpt $ case svc of
-    HttpOnly s  -> runSettingsSocket setting s app
+    HttpOnly s  -> runHTTP setting s app
 #ifdef HTTP_OVER_TLS
-    HttpsOnly s -> runTLSSocket tlsSetting setting s app
+    HttpsOnly s -> runHTTPS tlsSetting setting s app
     HttpAndHttps s1 s2 -> concurrently_
-        (runSettingsSocket setting s1 app)
-        (runTLSSocket tlsSetting setting s2 app)
+        (runHTTP setting s1 app)
+        (runHTTPS tlsSetting setting s2 app)
 #ifdef HTTP_OVER_QUIC
-    QUIC s1 s2 -> do
+    QUIC s1 s2 ss3 -> do
         let quicPort' = BS.pack $ show quicPort
             strver Q.Version1 = ""
             strver Q.Version2 = ""
@@ -203,10 +216,11 @@ mighty opt rpt svc lgr pushlgr mgr rdr _mcreds _msmgr tmgr
             value v = BS.concat ["h3",v,"=\":",quicPort',"\""]
             altsvc = BS.intercalate "," $ map value quicDrafts
             settingT = setAltSvc altsvc setting
-        mapConcurrently_ id [runSettingsSocket        setting  s1 app
-                            ,runTLSSocket  tlsSetting settingT s2 app
-                            ,runQUIC       qconf      setting     app
-                            ]
+            h12  = concurrently_ (runHTTP             setting  s1 app)
+                                 (runHTTPS tlsSetting settingT s2 app)
+            h123 = concurrently_ (runHTTP3 qconf      setting  ss3 app)
+                                 (labelMe "concurrently" >> h12)
+        h123
 #else
     _ -> error "never reach"
 #endif
@@ -215,12 +229,10 @@ mighty opt rpt svc lgr pushlgr mgr rdr _mcreds _msmgr tmgr
 #endif
   where
     app = fileCgiApp cspec filespec cgispec revproxyspec rdr
-    debug = opt_debug_mode opt
     -- We don't use setInstallShutdownHandler because we may use
     -- two sockets for HTTP and HTTPS.
     setting = setPort            (naturalToInt $ opt_port opt) -- just in case
             $ setHost            (fromString (opt_host opt))  -- just in case
-            $ setOnException     (if debug then printStdout else warpHandler rpt)
             $ setManager         tmgr
             $ setFdCacheDuration (naturalToInt $ opt_fd_cache_duration opt)
             $ setFileInfoCacheDuration 10
@@ -229,7 +241,7 @@ mighty opt rpt svc lgr pushlgr mgr rdr _mcreds _msmgr tmgr
             $ setServerPushLogger pushlgr
             defaultSettings
 #ifdef HTTP_OVER_TLS
-    tlsSetting = defaultTlsSettings {
+    ~tlsSetting = defaultTlsSettings {
         tlsCredentials    = _mcreds
       , tlsSessionManager = _msmgr
       , tlsAllowedVersions = [TLS.TLS13,TLS.TLS12]
@@ -251,10 +263,12 @@ mighty opt rpt svc lgr pushlgr mgr rdr _mcreds _msmgr tmgr
         revProxyManager = mgr
       }
 #ifdef HTTP_OVER_QUIC
-    quicAddr = read <$> opt_quic_addr opt
-    quicPort = fromIntegral $ opt_quic_port opt
-    quicVersions = Q.scVersions Q.defaultServerConfig
-    qconf = Q.defaultServerConfig {
+    ~quicAddr = read <$> opt_quic_addr opt
+    ~quicPort = fromIntegral $ opt_quic_port opt
+    ~quicVersions = Q.scVersions Q.defaultServerConfig
+    -- Lazy binding for opt_service == 0 so that
+    -- 'fromJust' is not called.
+    ~qconf = Q.defaultServerConfig {
             Q.scAddresses      = (,quicPort) <$> quicAddr
           , Q.scALPN           = Just chooseALPN
           , Q.scRequireRetry   = False
@@ -286,7 +300,13 @@ fromVersion (Q.Version ver) = fromIntegral (0x000000ff .&. ver)
 data Service = HttpOnly Socket
              | HttpsOnly Socket
              | HttpAndHttps Socket Socket
-             | QUIC Socket Socket
+             | QUIC Socket Socket [Socket]
+
+instance Show Service where
+    show HttpOnly{}     = "HttpOnly"
+    show HttpsOnly{}    = "HttpOnlys"
+    show HttpAndHttps{} = "HttpAndHttps"
+    show QUIC{}         = "QUIC"
 
 ----------------------------------------------------------------
 
@@ -294,34 +314,33 @@ openService :: Option -> IO Service
 openService opt
   | service == 1 = do
       s <- bindPortTCP httpsPort hostpref
-      debugMessage $ urlForHTTPS httpsPort
+      putStrLn $ urlForHTTPS httpsPort
       return $ HttpsOnly s
   | service == 2 = do
       s1 <- bindPortTCP httpPort hostpref
       s2 <- bindPortTCP httpsPort hostpref
-      debugMessage $ urlForHTTP httpPort
-      debugMessage $ urlForHTTPS httpsPort
+      putStrLn $ urlForHTTP httpPort
+      putStrLn $ urlForHTTPS httpsPort
       return $ HttpAndHttps s1 s2
   | service == 3 = do
       s1 <- bindPortTCP httpPort hostpref
       s2 <- bindPortTCP httpsPort hostpref
-      debugMessage $ urlForHTTP httpPort
-      debugMessage $ urlForHTTPS httpsPort
-      debugMessage "QUIC is also available via Alt-Svc"
-      return $ QUIC s1 s2
+      ss3 <- mapM (bindPortUDP quicPort) quicAddrs
+      putStrLn $ urlForHTTP httpPort
+      putStrLn $ urlForHTTPS httpsPort
+      putStrLn "QUIC is also available via Alt-Svc"
+      return $ QUIC s1 s2 ss3
   | otherwise = do
       s <- bindPortTCP httpPort hostpref
-      debugMessage $ urlForHTTP httpPort
+      putStrLn $ urlForHTTP httpPort
       return $ HttpOnly s
   where
     httpPort  = naturalToInt $ opt_port opt
     httpsPort = naturalToInt $ opt_tls_port opt
+    quicPort = naturalToInt $ opt_quic_port opt
+    quicAddrs = fromString <$> opt_quic_addr opt
     hostpref  = fromString $ opt_host opt
     service = opt_service opt
-    debug = opt_debug_mode opt
-    debugMessage msg = when debug $ do
-        putStrLn msg
-        hFlush stdout
     urlForHTTP  80  =  "http://localhost/"
     urlForHTTP  p   =  "http://localhost:" ++ show p ++ "/"
     urlForHTTPS 443 = "https://localhost/"
@@ -334,7 +353,7 @@ closeService :: Service -> IO ()
 closeService (HttpOnly s)         = close s
 closeService (HttpsOnly s)        = close s
 closeService (HttpAndHttps s1 s2) = close s1 >> close s2
-closeService (QUIC s1 s2)         = close s1 >> close s2
+closeService (QUIC s1 s2 ss3)     = close s1 >> close s2 >> mapM_ close ss3
 
 ----------------------------------------------------------------
 
@@ -350,14 +369,46 @@ getManager opt = H.newManager H.defaultManagerSettings {
       | opt_proxy_timeout opt == 0 = H.managerResponseTimeout H.defaultManagerSettings
       | otherwise                  = H.responseTimeoutMicro (naturalToInt $ opt_proxy_timeout opt * 1000000) -- micro seconds
 
-#ifdef DROP_EXCEPT_BIND
-foreign import ccall unsafe "send_signal"
-  c_send_signal :: CInt -> CInt -> IO ()
+----------------------------------------------------------------
 
-dropExceptBind :: IO ()
-dropExceptBind = do
-    pid <- getProcessID
-    strtids <- listDirectory ("/proc/" ++ show pid ++ "/task")
-    let tids = map read strtids :: [Int]
-    forM_ tids $ \tid -> c_send_signal (fromIntegral tid) sigUSR1
+setup :: Option -> IO (Maybe Credentials, Maybe SessionManager)
+#ifdef HTTP_OVER_TLS
+setup opt
+  | 1 <= service && service <= 3 = do
+        mcred <- Just <$> loadCredentials opt
+        smgr <- Just <$> SM.newSessionTicketManager SM.defaultConfig
+        return (mcred, smgr)
+  | otherwise = return (Nothing, Nothing)
+  where
+    service = opt_service opt
+#else
+setup _ = return (Nothing, Nothing)
+#endif
+
+labelMe :: String -> IO ()
+#if __GLASGOW_HASKELL__ >= 906
+labelMe lbl = do
+    tid <- myThreadId
+    labelThread tid lbl
+#else
+labelMe _ = return ()
+#endif
+
+runHTTP :: Settings -> Socket -> Application -> IO ()
+runHTTP setting s app = do
+    labelMe "HTTP1/2 server"
+    runSettingsSocket setting s app
+
+#ifdef HTTP_OVER_TLS
+runHTTPS :: TLSSettings -> Settings -> Socket -> Application -> IO ()
+runHTTPS tlsSetting setting s app = do
+    labelMe "HTTP1/2 over TLS server"
+    runTLSSocket tlsSetting setting s app
+
+#ifdef HTTP_OVER_QUIC
+runHTTP3 :: QUICSettings -> Settings -> [Socket]-> Application -> IO ()
+runHTTP3 qconf setting ss app = do
+    labelMe "HTTP3 server"
+    runQUICSockets qconf setting ss app
+#endif
 #endif
